package ma.game;

/* Esta clase contiene el manejo grfico del juego */

import java.awt.*;
import java.util.Vector;

import ma.game.*;

public class TCanvas extends Canvas
{
	// Para indicar si el canvas esta listo para dibujar.
	protected boolean i_canDraw = false;
	
	// Nombre del tipo de letra por defecto.
	private String is_FontName=null;
	
	// Referencia a la superficie grfica.
	private Image i_Surface = null;
	
	// Tamao del Surface.
	private Rectangle ir_Surface=null;
			
	// Referencia al tipo de letra por defecto.	
	protected TFont i_Fonts = null;
		
	// Color de fondo.
	protected Color miBackcolor = Color.white;	
			
	// Referencia al Applet
	public static TGame maGame = null;	
	
	// Inicializacin del Juego.
	//	
	public void init(String defFont,Image miSurface)
	{
		i_canDraw=false;
		
		// Establece el tipo de letra por defecto.
		setDefaultFont(defFont);
		
		// Inicializamos el resto de las cosas.
		this.init(miSurface);
	}		
	
	public void init(Image miSurface)
	{
		i_canDraw = false;
		
		// Establecemos el objeto surface.
		setSurface(miSurface);
				
		i_canDraw = (i_canDraw && (i_Surface!=null));		
	}		
	
	// Establece un tipo de letra.
	// Carga los tipos de letra.
	//
	public boolean setDefaultFont(String defFont)
	{
		is_FontName = defFont;
		i_Fonts = new TFont(is_FontName);
		return (i_Fonts!=null);
	}

	// Devuelve el tipo de letra asignado por defecto
	// al canvas.
	//
	public TFont getDefaultFont()
	{
		return i_Fonts;
	}
	
	// Establece un objeto surface que ser utilizado
	// como buffer.
	//
	public void setSurface(Image i)
	{
		i_Surface = i;
		ir_Surface = new Rectangle(0,0,i.getWidth(null),i.getHeight(null));
	}
	
	// Devuelve el controlador grafico de la superficio
	// del juego.
	//
	public Graphics getSurfaceGraphics()
	{
		return i_Surface.getGraphics();
	}

	// Devuelve el color de fondo de la superficie.
	//
	public void setSurfaceBackground(Color rgb)
	{
		miBackcolor = rgb;
		setBackground(rgb);
	}

	// Limpia toda la superficie del Canvas.
	//
	public void clear()
	{
	 clear(0,0,ir_Surface.width,ir_Surface.height);
	}
	
	// Limpia una regin de la superficie del Canvas.
	//	
	public void clear(int x,int y,int x1,int y1)
	{
	 Graphics g = i_Surface.getGraphics();	 
	 g.setColor(miBackcolor);	 
	 g.fillRect(x,y,x1,y1);
	}
	
	// Indica si el Canvas esta listo para poder dibujarse..
	//
	public boolean canDraw() 
	{ 
		return (i_canDraw &&  (maGame.i_Escenario!=null) ); 
	}	
	
	// Dibuja el Buffer grafico a la pantalla
	// que se indique.
	//
	void flip(Graphics g)
	{
		g.drawImage(i_Surface,0,0,null);
	}
		
	// Mtodo Update.
	//
	public void update(Graphics g)
	{
		draw(g);
	}
		
	// Mtodo Paint
	// Normalmente solo se lanza cuando se maximiza, minimiza o es
	// tapada o destapado por otra ventana.
	//
	public void paint(Graphics g)
	{
	  draw(g);
	}
	
	// Manejador de Eventos
	//
	public boolean handleEvent(Event evt)
	{
		// Redireccionamos el evento de pulsacin de cursores
		// a teclas de usuario.
		if (evt.id == Event.KEY_ACTION)
		{
			evt.id = Event.KEY_PRESS;
			evt.key = -evt.key;
		}								
		else if (evt.id == Event.KEY_ACTION_RELEASE)
		{
			evt.id = Event.KEY_RELEASE;
			evt.key = -evt.key;
		}
			
		// Si el escenario maneja el evento, salimos.
		if (maGame.i_Escenario!=null && maGame.i_Escenario.handleEvent(evt)) return true;
		
		// En otro caso que sigan manejandose los eventos 
		// definidos en el sistema por defecto.
		//
		return super.handleEvent(evt);
	}	
		
	// Loop principal para dibujar un escenario.
	//
	synchronized protected void draw(Graphics g)
	{
		Graphics g1;	
		
		if (!canDraw()) return;
		
		i_canDraw = false;
		
		g1 = i_Surface.getGraphics();		
		maGame.i_Escenario.draw(g1);
		flip(g);
		
		i_canDraw = true;
		
		// Caso de la tecla Abortar.
		//
		//if (maGame.i_Teclas[5][1]==1) maGame.startClass();
		
	}
	
	protected void drawLoading()
	{
			maGame.drawLoading(getGraphics());			
			// Hacemos una pausa de medio segundo para que
			// le de tiempo al avento paint a lanzarse.
			//
			try { Thread.currentThread().sleep(500); }
			catch (Exception e) {} ;
	}
	
		
}