package ma.game;

// Esta clase debe implementar el comportamiento de
// un escenario determinado en el juego.

import java.awt.*;
import java.util.Vector;

import ma.game.*;
import ma.utils.*;

public class TEscenario
{
	public static TGame maGame = null;
	
	protected TFont			miFont = null;
	protected int				i_Score = 0;	
	protected int				i_Estado = 0;
		
	protected Vector i_Sprites = new Vector();
	protected Vector i_ColisionesMalos = new Vector();
	protected Vector i_ColisionesBuenos = new Vector();
	protected TZBufferList i_ZBuffer = new TZBufferList(100,20);
	protected TZBufferEnumeration i_ZBufferEnum = new TZBufferEnumeration(i_ZBuffer,false);
	
	// Constructor del Escenario
	//
	public TEscenario() 
	{
		Constructor(null);
	}
	
	public TEscenario (TFont deffont)
	{
	 Constructor(deffont);	
	}
		
	// Mtodo de Construccin avanzado.
	//
	public void Constructor(TFont deffont)
	{		
		// Establece el font por defecto.
		if (miFont==null) 
		{
			miFont = maGame.i_gameCanvas.getDefaultFont();
		}
		else miFont = deffont;
	}
	
	// El manejador de Evento ha de ser definido 
	// por cada subclase
	//
	public boolean handleEvent(Event evt)
	{
		return false;
	}
	
	// Pinta algo sobre el escenario, en caso de tener
	// que hacerlo, por defecto no se hace nada.
	//
	public void draw(Graphics g)
	{	}
	
	// Resetea el Escenario para que empiece de nuevo.
	//
	public void reset()
	{
		i_Estado = 0;
		i_Score = 0;
	}
	
	// Inicializacin del Escenario.
	// Debe ser implementado por las subclases
	// En caso que sea necesario.
	//
	public void init()
	{}
	
	// Resetea el Vector de Sprites.
	//
	public void SpriteReset()
	{
		while (i_Sprites.size()>0)
		{
			i_Sprites.removeElementAt(0);
		}
	}
	
	// Carga una imagen como sprite y devuelve la 
	// el indice del grfico.
	//
	public int loadSpriteImage(String name)
	{
		i_Sprites.addElement(maGame.getTracker().AddWaitImage(maGame.LoadImage(name)));
		return i_Sprites.size()-1;
	}
	
	// Devuelve la imagen de un sprite.
	//
	public Image getSpriteImage(int i)
	{
		return (Image)i_Sprites.elementAt(i);
	}
	
	// Aade un Sprite a una capa.
	//
	public void addSprite(int i,TSprite S)
	{
		i_ZBuffer.addItem(S,i);
		
		// Introducimos algunos sprites en determinadas listas
		// para detectar mejor las colisiones.
		//
		if (S instanceof TEnemigo)	i_ColisionesMalos.addElement(S);
		if (S instanceof TAliado)	i_ColisionesBuenos.addElement(S);
	}
	
	// Elimina un Sprite de las listas de Sprites.
	//
	public void removeSprite(TSprite S)
	{
		i_ZBuffer.removeItem(S);
		if (S instanceof TEnemigo) i_ColisionesMalos.removeElement(S);
		if (S instanceof TAliado)  i_ColisionesBuenos.removeElement(S);
	}
	
	// Pinta los personajes o Sprites situados en las capas
	// Existentes.
	// La capa 0 es la ltima en Pintarse.
	// La capa ms grande es la primera.
	//
	public void drawSprites(Graphics g)
	{		
		TSprite S;
		i_ZBufferEnum.Reset();
		
		while (i_ZBufferEnum.hasMoreElements())
		{
			S = (TSprite)i_ZBufferEnum.nextElement();
				
			if (!S.play())
			{
				S.finish();
				removeSprite(S);				
			}
			else 
			{
				if (!S.selfDraw(g))
				{
					Image img = (Image)i_Sprites.elementAt(S.curFrame);
					g.drawImage(img,S.pos.x,S.pos.y,null);
					S.pos.width = img.getWidth(null);
					S.pos.height = img.getHeight(null);
				}
			}			
		} // Fin del while
		
	}
	
}