package ma.game;

import java.awt.*;
import java.applet.Applet;
import java.util.Vector;
import java.util.Date;

import ma.game.*;
import ma.TMsgBox;

// Clase TGame.
// Clase principal del juego, extendida de Applet.

public class TGame extends Applet implements Runnable
{
    // Referencia al Canvas principal.
		public static final TCanvas i_gameCanvas = new TCanvas();
	
		// Referencia a la instancia del escenario.
		public static TEscenario i_Escenario;
	
		// Control de Teclas..
		public static int i_Teclas[][] = new int[6][2]; // Der, Izq, Arr, Aba, Disp, Abort.
	
		protected Thread i_Thread= null;
		protected int i_GameSpeed = 40;
	  protected Vector i_constSpr = new Vector();	  
		protected String is_gamePath;
	  
		// Media Tracker de Imgenes asociado al juego.
		protected TImageTracker i_Tracker;
	
		// Nombre de la clase que ser el menu del juego.
		// Ha de heredar de TEscenario.
		private String is_StartClass=null;
		
		// Constructor
		//
		public TGame()
		{			
			// Inicializamos las clases apuntando a la instancia del juego.
			//
			TSprite.maGame = this;
			TImageTracker.maGame = this;
			TFont.maGame = this;
			TEscenario.maGame = this;
			TCanvas.maGame = this;
			
			i_Tracker = new TImageTracker();		
			setLayout(new BorderLayout());
			add("Center",i_gameCanvas);	
			is_gamePath="";
			
			// Definicin de las teclas por defecto...
		
			i_Teclas[0][0] = 'p'; // Derecha
			i_Teclas[1][0] = 'o'; // Izquierda
			i_Teclas[2][0] = 'q'; // Arriba
			i_Teclas[3][0] = 'a'; // Abajo
			i_Teclas[4][0] = ' '; // Disparo
			i_Teclas[5][0] =  27; // Abortar				

    } 
				
		// Asigna un escenario al Juego.
		//
		public void AsignaEscenario(TEscenario escena)
		{			
			i_gameCanvas.i_canDraw = false;
			
			// Pintamos el texto de carga.. si procede.
			i_gameCanvas.drawLoading();
			
			if (i_Escenario!=null)
			{
				i_Escenario = null;
				System.gc();
			}
			i_Escenario = escena;
			
			// Inicializa el escenario.
			i_Escenario.init();
			
			i_gameCanvas.i_canDraw = true;
		}
	
		// Devuelve una referencia al Tracker
		//
		public TImageTracker getTracker()
		{
			return i_Tracker;
		}
	
		// Devuelve una imagen cargada a traves del applet.
		//
		public Image LoadImage(String name)
		{
			Image img = null;
			try
			{
			 // Intentamos hacer la carga desde un applet.
			 img = getImage(getDocumentBase(),is_gamePath+"/"+name);
			}
			catch (Exception e) 
			{			
			   // Intentamos hacer la carga desde la shell.
				 img = getToolkit().getImage(name);
				
			}
			return img;
		}
		
		// Copia referencias de Sprites Constantes a otro Vector.
		//
		public void copyConstSpr(int ini,int fin,Vector V)
		{
			for (int i=ini;i<=fin;i++)
			{
				V.addElement(i_constSpr.elementAt(i));
			}
		}				

		// Este mtodo es invodado por la hebra
		// Cuando se arranca con start.
		//
	  public void run()
		{
			long ll_wait;
			long ll_wait0,ll_wait1;
			long ll_itera_gc=0;
			ll_wait0 =  System.currentTimeMillis();
			 
			do
			{
			 ll_wait1 =  System.currentTimeMillis();			 	
		
			 if (ll_wait1 - ll_wait0 > i_GameSpeed) 
			 {			 
			  i_gameCanvas.paint(i_gameCanvas.getGraphics());
			  ll_wait0 =  System.currentTimeMillis();			
			 }
				
			 /*
			 ll_itera_gc++;
			 if (ll_itera_gc==100)
			 {
					ll_itera_gc = 0;
					System.gc();
			 }
			 */
					
			} while (true);
		}
	
		// Establece la velocidad del retardo del Juego
		public void setGameSpeed(int speed)
		{
			i_GameSpeed = 1000 / speed;
		}				
	
	
		// Este mtodo lo ejecuta el applet cuando
		// restauran la ventana del navegador.
		//
    public void start()
    {
    	if (i_Thread!=null) i_Thread.resume();
    } 
	
    
		// Este mtodo lo ejecuta el applet cuando
		// se minimiza o cierra la ventana del navegador.
		//
    public void stop()
    {
    	if (i_Thread!=null) i_Thread.suspend();
    }
		
		// Devuelve el objeto graphics de la superficie del Canvas.
		//
		public Graphics getSurfaceGraphics()
		{
			return i_gameCanvas.getSurfaceGraphics();
		}

		// Establece el nombre de la calse inicial.
		//
		public void setStartClass(String sc)
		{
			is_StartClass = sc;
		}
		
		// Arranca la Clase especificada como principal.
		// Inicializa el Escenario para que aparezca el Menu del Juego.
		//
		public void startClass()
		{
			TEscenario lmenu;
			
			// Liberamos la tecla de Abortar..
			i_Teclas[5][1]=0;
			
			try
			{
				// Creamos dinamicamente la clase escenario.
				Class miClase;			
				miClase = Class.forName(is_StartClass);
				lmenu = (TEscenario) miClase.newInstance();				
				lmenu.Constructor(i_gameCanvas.getDefaultFont());
		
				// Asigna el escenario.
				AsignaEscenario(lmenu);
				
			}
			catch (Exception e)
			{
				System.out.println("No se pudo cargar la clase: " + is_StartClass+". " + e);
				new TMsgBox("Atencin","No se pudo cargar la clase: " + is_StartClass);
			}
		}	
	
	// Carga un sprite Constante en la lista de Sprites Constantes.
	//
	public int loadConstSprImage(String name)
	{
		i_constSpr.addElement(i_Tracker.AddWaitImage(this.LoadImage(name)));
		return i_constSpr.size()-1;
	}		
	
	// Devuelve la imagen de un sprite constante.
	//
	public Image getConstSprImage(int i)
	{
		return (Image)i_constSpr.elementAt(i);
	}
	
	// Este evento lo invoca el Canvas cuando
	// no puede dibujar..
	//
	public void drawLoading(Graphics g)
	{}

} // End of Class 