package ma.utils;

import java.util.Enumeration;
import ma.utils.TZBufferItem;

public class TZBufferList
{
	protected TZBufferItem i_Buff[];
	private int ii_count=0;
	private int ii_delta;
	private int ii_actual_size;
	private int ii_original_size;
	
	// Constructor de una lista con cabida de 100
	// elementos y con un incremento de 10 cuando 
	// se acabe el espacio.
	//
	public TZBufferList()
	{
		this(100,10);
	}
	
	// Constructor donde se puede especificar el
	// tamao inicial y el tamao del incremento.
	//
	public TZBufferList(int size,int delta)
	{
		Reset(size,delta);
	}
	
	// Elimina todos los elementos de la lista
	// y reestablece los parametros con que se creo.
	//
	public void Reset()
	{
		Reset(ii_original_size,ii_delta);
	}
	
	private void Reset(int size,int delta)
	{
	  ii_delta = delta;
		ii_original_size = size;
		ii_actual_size = size;
		i_Buff = new TZBufferItem[size];
		System.gc();
	}
	
	// Limpia la lista.
	//
	public void Clear()
	{
		for (int i=0;i<ii_count;i++)
		{
			i_Buff[i]=null;
		}
		System.gc();
		ii_count = 0;
	}
	
	// Devuelve el numero de elementos almacenado
	// en la lista.
	//
	public int getCount()
	{
		return ii_count;
	}
	
	private TZBufferItem insertItem(int pos,Object I,int zorder)
	{
		// Comprobamos que haya capacidad para insertar.
		// Si no es as ampliamos el array.
		if (ii_count == ii_actual_size)
		{
			TZBufferItem lbuff[] = new TZBufferItem[ii_actual_size + ii_delta];
			System.arraycopy(i_Buff,0,lbuff,0,ii_actual_size);
			i_Buff = lbuff;
			lbuff=null;
			ii_actual_size += ii_delta;
			System.gc();
		}
		
		// Ahora Insertamos.
		TZBufferItem l_item = new TZBufferItem(I,zorder);
		System.arraycopy(i_Buff,pos,i_Buff,pos+1,ii_actual_size - pos - 1);
		i_Buff[pos]=l_item;
		ii_count++;
		
		return l_item;
			
	}
	
	// Inserta un item ordenandolo por el zorder.
	// * Devuelve una referencia al TZBufferItem que
	//   contiene informacin del elemento.
	// * Si de vuelve null elemento no se inserto.
	//
	synchronized public TZBufferItem addItem(Object I,int zorder) 
	{
		TZBufferItem l_item;
		int li_begin,li_end,li_pos;
		
		// Caso de que sea el primer elemento.
		//
		if (ii_count==0)
		{
			i_Buff[ii_count]=new TZBufferItem(I,zorder);
			return i_Buff[ii_count++];
		}
		
		// Hay que buscar el hueco.
		//
		li_begin = 0;
		li_end = ii_count-1;
		
		do
		{
			li_pos = (li_begin+li_end)/2;
			
			// Tomamos el elemento en la posicin
			// indicada.
			l_item = i_Buff[li_pos];
			
			if (zorder >= l_item.zorder)
			{
				// Hay que insertar en la mitar superior.
				li_begin = li_pos + 1;
			}
			else if (zorder < l_item.zorder)
			{
				// Hay que insertar en la mitad inferior.
				li_end = li_pos - 1;
			}
			
		} while (li_begin <= li_end);
			
		// Hay que insertar el elemento.	
		if (zorder>l_item.zorder) li_pos++;
		
		return insertItem(li_pos,I,zorder);
	}
	
	// Elimina un Item de la lista.
	//
	synchronized public boolean removeItem(Object I) 
	{
		int li_i;
		
		for(li_i=0;li_i<ii_count;li_i++)
		{
			if (i_Buff[li_i].item == I) 
			{
				System.arraycopy(i_Buff,li_i+1,i_Buff,li_i,ii_count - li_i - 1);
				i_Buff[ii_count]=null;
				ii_count --;
				return true;
			}
		}
		return false;
	}
		
	
	// Devuelve un iterador ascendente de la lista.
	//
	synchronized public Enumeration upEnumeration()
	{
		return new TZBufferEnumeration(this,true);
	}
	
	// Devuelve un iterador descendente de la lista.
	//
	synchronized public Enumeration downEnumeration()
	{
		return new TZBufferEnumeration(this,false);
	}
	
	public String toString()
	{
		return "TZBufferList("+ ii_count +"," +
													  ii_delta +"," +
														ii_actual_size +","+
														ii_original_size+")";
	}
	
}